;-------------------------------------------------------------------------------------
;Charlieplexing 20 LEDs
;----------------------
;Fr ATtiny 13, 4.8 MHz interner R/C Oszilator
;Die 20 LEDs sind als 4*5 Matrix angeordnet und knnen Laufschriften und Leuchtmuster
;anzeigen, die als Bitmap vorliegen. 
;
;Autor: Jrgen Woetzel, juergen@avr-projekte.de
;Version 1.0 vom 30.10.2013 
;***********************************************************************************
.include "tn13def.inc"
;***********************************************************************************
;Registerdefs
;***********************************************************************************
.def    blow    =   r0
.def    bhigh   =   r1
.def    w10     =   r2
.def    tick    =   r3
.def	s		=	r5
.def	itemp1	=	r6
.def    arrayl  =   r8
.def    arrayh  =   r9
.def    mode    =   r10
.def    map1    =   r12
.def    map2    =   r13
.def    map3    =   r14
.def    mux     =   r15
.def	temp1 	= 	r16
.def    temp2   =   r17
.def    temp3   =   r18
.def    temp4   =   r19
.def    xpos    =   r21
.def    icnt    =   r22
.def    with    =   r23
.def    pix     =   r24
.equ    scroll  =   1
.equ    film    =   4
;***********************************************************************************
;Makros
;***********************************************************************************
.macro play
        ldi     zl,low(@0*2)  ;Anfangsadresse der Bitmap im Flash
        ldi     zh,high(@0*2)
        rcall   player     
.endm
;***********************************************************************************
;SRam
;***********************************************************************************
.dseg
.org    $60
array:	.byte	40  ;40 Bytes fr die Charlieplexing Daten reservieren
.cseg
;***********************************************************************************
;Sprungvektoren
;***********************************************************************************
.org    0
        rjmp	init
.org    OVF0addr
;***********************************************************************************
;Timerinterrupt ca. alle 0,42 mS.
;***********************************************************************************
        in      s,sreg              ;SREG wegspeichern
        dec     tick                ;Alle 0,42 mS (Variable fr Unterprogramm wait)
        dec     icnt                ;Zhler fr Multipex
        brne    xx1                 ;Wenn Muxzhler =0, Muxzhler neu laden und Zeiger auf erste LED
        mov     icnt,mux
        ldi     yl,array            ;YH ist immer 0 (Array = $60..$88)
xx1:    ld      itemp1,y+           ;Daten fr Tristate und Ports der gerade aktiven LED 
        out     ddrb,itemp1         ;in die C Register schreiben
        ld      itemp1,y+           
        out     portb,itemp1
        out     sreg,s              ;SREG zurck
        reti   
;***********************************************************************************
;Initialisierung bei Power on
;***********************************************************************************
init:   ldi     temp1,LOW(RAMEND)   ;Stack
        out     SPL,temp1
        ;---------------------------------------------------------------------------
        ;Timeroverflow Interrupt anwerfen
        ;---------------------------------------------------------------------------
        ldi     temp1,1<<cs01       ;Prescaler /8
        out     TCCR0B,temp1        ;4,8MHz/256/8= 2,35KHz. ISR alle 0,42 mS
        ldi     temp1, 1<<toie0     ;Interrupt bei Timer Overflow ein
        out     TIMSK0, temp1
        ;---------------------------------------------------------------------------
        ;Programminit
        ;---------------------------------------------------------------------------
        clr     yh                  
        clr     xpos
        rcall   clrarray            ;Alle LEDs aus
        sei
        ;**************************************************************************
        ;Main
        ;**************************************************************************
main:   play    laufschrift
        play    snake   
        play    text
        play    pong
        play    anim
        play    quick
        rjmp    main
        ;**************************************************************************
        ;Main Ende
        ;**************************************************************************
        ;----------------------------------------------------------------------------
        ;Bitmap durch das Display schieben
        ;----------------------------------------------------------------------------
player: lpm     mode,z+         ;Wird in der Schleife zur X-Position addiert (1 oder 4)
        lpm     pix,z+          ;Breite der Bitmap in Pixel
        mov     with,pix        ;Breite der Bitmap in Byte (Pixel/8)
        lsr     with
        lsr     with
        lsr     with
        brne    sc0             ;Passiert nur bei 256px breite
        ldi     with,256/8
sc0:    lpm     mux,z+
        lpm     w10,z+          ;Zeit zwischen den Einzelnen Frames in ms*10
        movw    blow:bhigh,z    ;Start der Bitmap wegspeichern
        clr     xpos            ;XPosition=0
sc1:    movw    z,blow:bhigh
        rcall   bitm            ;Frame (20 Anzuzeigende LEDs) aus der Bitmap herauskopieren
        rcall   movbitm         ;Aus Herauskopierten Daten Charlieplexliste erstellen
        rcall   wait            ;Zeit vertrdeln
        add     xpos,mode       ;X-Position um 1 (Scrolling) oder 4 (Daumenkino) erhhen
        cp      xpos,pix        ;Sind wir am Ende der Map ?
        brne    sc1             ;Wenn nein, Sprung
        ret
        ;----------------------------------------------------------------------------
        ;Kopiert die nchsten 20 Pixel von der Bitmap (xpos) nach map1..3
        ;----------------------------------------------------------------------------
bitm:   ldi     temp4,5         ;5 Zeilen
        ;----------------------
        ;Adresse berechnen. 
        ;----------------------
        mov     temp1,xpos      ;Das richtige Byte ist Anfangsadr. +(xpos/8)
        lsr     temp1           ;temp1 = xpos/8 
        lsr     temp1
        lsr     temp1
        add     zl,temp1        ;Offset addieren
        adc     zh,yh
bi5:    lpm     temp1,z+        ;Die nchsten 2 Byte aus der Bitmap holen
        lpm     temp3,z
        ;-------------------------------
        ;Die 4 Bits in Position schieben
        ;-------------------------------
        mov     temp2,xpos      ;Position inerhalb des Bytes ist in den unteren 3 Bits von xpos
        andi    temp2,0b111 
bi1:    tst     temp2           ;Um die Anzahl der unteren 3 Bits (0..7) in xpos (temp2)...
        breq    bi3             ;nach links schieben
        rol     temp3
        rol     temp1
        dec     temp2
        rjmp    bi1
        ;----------------------
        ;die 4 Bits kopieren 
        ;----------------------
bi3:    ldi     temp2,4         ;Die nchsten 4 Bit in map3..1 schieben
bi4:    rol     temp1           ;Nach 5 Zeilen sind die 20 Bit rechtsbndig in map1..3
        rol     map3
        rol     map2
        rol     map1
        dec     temp2
        brne    bi4
        dec     temp4           ;Nchste Zeile
        breq    bi6
        ;----------------------
        ;Z auf nchste Zeile
        ;----------------------
        mov     temp2,with      ;Bitmapbreite in Bytes nach temp2
        dec     temp2           ;Da Z einmal erhht wurde, -1
        add     zl,temp2        ;Zum Bitmapzeiger addieren
        adc     zh,yh           ;Carry mitaddieren (yh ist null)
        rjmp    bi5             ;Nchste Runde
bi6:    ret             
        ;--------------------------------------------------------------------       
        ;Die 4*5 Bitmap in map3..1 als Charlieplexdaten im SRam ablegen
        ;--------------------------------------------------------------------       
movbitm:rcall   clrarray            ;Alle LEDs aus, X steht danach auf Arrayanfang.
        ldi     zl,low(CharlieTab*2);Zeiger der Charlietab im Flash
        ldi     zh,high(CharlieTab*2);20 Datenpaare (DDRB,PortB) der LEDs 1..20
        ldi     temp4,20            ;Zhler. Anzahl der Datenpaare       
mo2:    lpm     temp1,z+            ;LED1, DDRB
        lpm     temp2,z+            ;LED1, PortB
        rol     map3                ;20 Bits schieben
        rol     map2            
        rol     map1                ;LED1-Bit ins Halfcarry. Die 20 Bit stehen rechtsbndig in map1..3
        brhs    mo1                 ;Wenn LED1 auf 1 (Halfcarry gesetzt), Sprung (sbrs map1,4 wrde auch klappen)
        rjmp    mo3                 ;Ansonsten Daten fr LED1 verwerfen (LED1 leuchtet nicht)
mo1:    st      x+,temp1            ;DDRB und PortB im SRam speichern (LED1 leuchtet)
        st      x+,temp2
mo3:    dec     temp4               ;Tabellenindex herunterzhlen
        brne    mo2                 ;Nach 20 Durchgngen beenden
mo4:    ret
        ;--------------------------------------------------------------------       
        ;SRam Array lschen
        ;--------------------------------------------------------------------       
clrarray:ldi     xl,array+40        ;Von hinten lschen, damit am Ende X auf Arrayanfang steht;-)
        clr     xh
        ldi     temp4,40            ;40 Bytes zu lschen
cl1:    st      -x,yh               ;Und Los (yh ist null)
        dec     temp4
        brne    cl1                 ;Das ganze 40 mal
        ret
        ;--------------------------------------------------------------------       
        ;Zeit vertrdeln
        ;--------------------------------------------------------------------       
wait:   mov     temp1,w10
wait1:  rcall   w10ms
        dec     temp1
        brne    wait1
        ret
        ;--------------------------------------------------------------------       
        ;10 mS warten
        ;--------------------------------------------------------------------       
w10ms:  push    temp1
        ldi     temp1,24      ;ca. 10mS
        mov     tick,temp1
wt:     tst     tick
        brne    wt
        pop     temp1
        ret
;**************************************************************************************
CharlieTab:
    .db 0b10001,0b00001,0b10001,0b10000
    .db 0b00011,0b00001,0b00011,0b00010
    .db 0b00101,0b00001,0b00101,0b00100
    .db 0b01001,0b00001,0b01001,0b01000
    .db 0b10010,0b10000,0b10010,0b00010
    .db 0b10100,0b10000,0b10100,0b00100
    .db 0b11000,0b10000,0b11000,0b01000
    .db 0b00110,0b00010,0b00110,0b00100
    .db 0b01010,0b00010,0b01010,0b01000
    .db 0b01100,0b01000,0b01100,0b00100
;**************************************************************************************
;Ab hier knnen Bitmaps eingefgt werden
;**************************************************************************************
/*
	Datei: lauf.bmp
	gre: 88x5
*/
Laufschrift:
.db scroll,88,13,15    ;Modus, Breite in Pixel, Mux, Wait
.db 0x31, 0x18, 0x9B, 0x8E, 0xA5, 0xC4, 0x25, 0x73, 0x35, 0x65, \
    0xDC, 0x2A, 0xA0, 0xA1, 0x08, 0xB5, 0x04, 0x55, 0x44, 0x45, \
    0x55, 0x08, 0x2A, 0x90, 0x91, 0x0C, 0xAD, 0x84, 0x55, 0x62, \
    0x47, 0x65, 0x88, 0x2B, 0x88, 0x89, 0x08, 0xA5, 0x04, 0x75, \
    0x41, 0x45, 0x55, 0x08, 0x32, 0xB0, 0xB1, 0x0E, 0xA5, 0xC7, \
    0x52, 0x46, 0x35, 0x55, 0x08, /*padding*/ 0
/*
	Datei: allesgute.bmp
	gre: 144x5
*/
text:

.db scroll,144,13,15    ;Modus, Breite in Pixel, Mux, Wait
.db 0x09, 0x11, 0xCC, 0x1A, 0xBB, 0x87, 0x54, 0x41, 0xBB, 0x2B, 0x39, 0xB9, 0x18, 0x00, 0xD4, 0x0C, 0x80, 0x00, \
    0x15, 0x11, 0x10, 0x22, 0x92, 0x01, 0x56, 0xC2, 0x22, 0xAA, 0x92, 0x12, 0xA0, 0x00, 0x41, 0x55, 0x54, 0x00, \
    0x15, 0x11, 0x88, 0x22, 0x93, 0x02, 0x55, 0x42, 0x33, 0x2B, 0x11, 0x12, 0xA0, 0x00, 0x55, 0x8D, 0x9A, 0x00, \
    0x1D, 0x11, 0x04, 0x2A, 0x92, 0x04, 0x54, 0x42, 0xA2, 0xAA, 0x90, 0x93, 0xA8, 0x01, 0x55, 0x05, 0x12, 0x00, \
    0x15, 0xDD, 0xD8, 0x19, 0x13, 0x87, 0x24, 0x41, 0xBB, 0x12, 0x93, 0x12, 0x98, 0x00, 0x8D, 0x18, 0xD2, 0x54
/*
	Datei: anim.bmp
	gre: 256x5
*/
anim:
.db film,0,14,8
.db 0x08, 0xCE, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xF7, 0x31, 0x00, \
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xF3, 0x9C, 0xEF, 0xFF, \
    0xFF, 0xFF, 0xFF, 0xFF, 0xF0, 0x0F, 0xF0, 0x00, 0x0F, 0x00, \
    0x00, 0x00, 0x00, 0x00, 0x01, 0x11, 0x11, 0x11, 0x11, 0x99, \
    0x99, 0x98, 0x88, 0x88, 0x88, 0x88, 0x00, 0x06, 0x99, 0x99, \
    0x88, 0x99, 0x99, 0x99, 0x91, 0x19, 0x96, 0x69, 0x96, 0x60, \
    0xF0, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x11, 0x11, \
    0x19, 0x99, 0x99, 0x99, 0x88, 0x88, 0x88, 0x80, 0x09, 0x66, \
    0x99, 0x99, 0x98, 0x89, 0x99, 0x99, 0x11, 0x99, 0x96, 0x69, \
    0x96, 0x6F, 0x00, 0x0F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, \
    0x11, 0x11, 0x99, 0x99, 0x99, 0x99, 0x98, 0x88, 0x88, 0x00, \
    0x00, 0x06, 0x99, 0x99, 0x99, 0x88, 0x99, 0x91, 0x19, 0x99, \
    0x96, 0x69, 0x96, 0x60, 0x00, 0x00, 0xF0, 0x00, 0x00, 0x00, \
    0x00, 0x00, 0x13, 0x7F, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xEC, \
    0x80, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF, 0xFE, 0xC9, 0x37, \
    0xFF, 0xFF, 0xF0, 0x0F, 0xF0, 0x00, 0x00, 0x00, 0x0F, 0x00
/*
	Datei: pong.bmp
	gre: 200x5
*/
pong:
.db film,200,11,20
.db 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xDD, 0xDD, \
    0xDD, 0xDD, 0xDD, 0x99, 0x99, 0x9D, 0x99, 0x99, \
    0x99, 0x11, 0x11, 0x19, 0x11, 0x11, 0x11, 0x00, \
    0x00, 0xFF, 0xEE, 0xEA, 0xAA, 0xA2, 0x26, 0x62, \
    0x22, 0x20, 0x00, 0x80, 0x80, 0x00, 0x80, 0x20, \
    0x00, 0x80, 0x40, 0x00, 0x40, 0x40, 0x00, 0x10, \
    0x20, 0x00, 0x01, 0x20, 0x82, 0x04, 0x00, 0x80, \
    0x08, 0x02, 0x01, 0x04, 0x00, 0x04, 0x04, 0x00, \
    0x01, 0x04, 0x00, 0x02, 0x02, 0x00, 0x02, 0x01, \
    0x00, 0x04, 0x00, 0x20, 0x04, 0x00, 0x10, 0x04, \
    0x00, 0x00, 0x40, 0x00, 0x20, 0x00, 0x00, 0x20, \
    0x00, 0x00, 0x20, 0x00, 0x00, 0x10, 0x00, 0x00, \
    0x10, 0x00, 0x00, 0x80, 0x63, 0x36, 0xC6, 0x33, \
    0x33, 0x66, 0x66, 0x66, 0x63, 0x66, 0x66, 0x63, \
    0x33, 0x33, 0x33, 0x66, 0x66, 0x33, 0x33, 0x33, \
    0x63, 0x36, 0x66, 0x6C, 0xCC, /*padding*/ 0
/*
	Datei: snake2.bmp
	gre: 240x5
*/
snake:
.db film,240,8,25
.db 0x00, 0x00, 0x00, 0x00, 0x02, 0x64, 0x00, 0x00, 0x26, 0xEC, \
    0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x8C, 0xEE, 0x62, 0x08, \
    0x88, 0x9B, 0xFF, 0xFE, 0xC8, 0x00, 0x00, 0x00, 0x00, 0x00, \
    0x00, 0x00, 0x02, 0x22, 0x22, 0x04, 0x42, 0x22, 0x22, 0x08, \
    0x88, 0x00, 0x00, 0x00, 0x00, 0x08, 0x88, 0x82, 0x22, 0x20, \
    0x01, 0x11, 0x19, 0x88, 0x88, 0x82, 0x22, 0x22, 0x22, 0x00, \
    0x00, 0x84, 0x00, 0x00, 0x20, 0x00, 0x44, 0x02, 0x20, 0x00, \
    0x88, 0x80, 0x00, 0x00, 0x02, 0xAA, 0xAA, 0x22, 0x22, 0x22, \
    0x11, 0x11, 0x10, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, \
    0x08, 0x00, 0x40, 0x02, 0x00, 0x00, 0x04, 0x62, 0x00, 0x00, \
    0x2A, 0xEE, 0x77, 0x31, 0x08, 0x88, 0x80, 0x00, 0x22, 0x23, \
    0x31, 0x11, 0x00, 0x08, 0x88, 0x88, 0x80, 0x00, 0x00, 0x00, \
    0x80, 0x00, 0x04, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, \
    0x00, 0x00, 0x01, 0x37, 0xFE, 0xC8, 0x00, 0x00, 0x02, 0x33, \
    0x33, 0x10, 0x00, 0x00, 0x9D, 0xFF, 0xFF, 0x73, 0x10, 0x00
/*
	Datei: the quick.bmp
	gre: 168x5
*/
quick:
.db scroll,168,12,20
.db 0x1D, 0x5C, 0x25, 0x4D, 0x46, 0x62, 0x45, 0x23, 0x92, 0x8E, \
    0xA8, 0xB1, 0x84, 0x8B, 0xB0, 0xEA, 0xE2, 0x13, 0xD4, 0x62, \
    0x30, 0x09, 0x50, 0x55, 0x51, 0x45, 0x55, 0x45, 0xA2, 0x2A, \
    0x82, 0xAD, 0xAA, 0x0A, 0x8A, 0x28, 0x4A, 0x82, 0x28, 0x54, \
    0x55, 0x40, 0x09, 0xD8, 0x55, 0x51, 0x86, 0x65, 0x55, 0x63, \
    0x29, 0x02, 0xAA, 0xB1, 0x0A, 0x8B, 0x30, 0x4E, 0xC2, 0x28, \
    0x94, 0x55, 0x40, 0x09, 0x50, 0x55, 0x51, 0x45, 0x55, 0x55, \
    0x22, 0x2A, 0x8A, 0xA8, 0xA0, 0x8A, 0x52, 0x28, 0x4A, 0x82, \
    0x39, 0x08, 0x55, 0x50, 0x09, 0x5C, 0x32, 0x4D, 0x46, 0x52, \
    0x29, 0x22, 0x12, 0x84, 0x48, 0xA3, 0x04, 0x23, 0xA8, 0x4A, \
    0xE3, 0xAB, 0xC8, 0x62, 0x30, /*padding*/ 0
